<?php

namespace Forms\Leave\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Forms\Leave\Http\Requests\LeaveRequest;
use Forms\Leave\Interfaces\LeaveRepositoryInterface;
use Forms\Leave\Models\Leave;
use Forms\Leave\Forms\Leave as Form;

/**
 * APIController  class for leave.
 */
class LeaveAPIController extends BaseController
{

    /**
     * Initialize leave resource controller.
     *
     * @param type LeaveRepositoryInterface $leave
     *
     * @return null
     */
    public function __construct(LeaveRepositoryInterface $leave)
    {
        parent::__construct();
        $this->repository = $leave;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Forms\Leave\Repositories\Criteria\LeaveResourceCriteria::class);
    }

    /**
     * Display a list of leave.
     *
     * @return Response
     */
    public function index(LeaveRequest $request)
    {
        return $this->repository
            ->setPresenter(\Forms\Leave\Repositories\Presenter\LeavePresenter::class)
            ->paginate();
    }

    /**
     * Display leave.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function show(LeaveRequest $request, Leave $leave)
    {
        return $leave->setPresenter(\Forms\Leave\Repositories\Presenter\LeaveListPresenter::class);
        ;
    }

    /**
     * Create new leave.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LeaveRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('leave::leave.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('leave/leave/' . $leave->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('leave/leave');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the leave.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function update(LeaveRequest $request, Leave $leave)
    {
        try {
            $data = $request->all();

            $leave->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('leave::leave.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('leave/leave/' . $leave->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('leave/leave/' . $leave->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the leave.
     *
     * @param Model   $leave
     *
     * @return Response
     */
    public function destroy(LeaveRequest $request, Leave $leave)
    {
        try {
            $leave->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('leave::leave.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('leave/leave/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('leave/leave/' . $leave->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
