<?php

namespace Forms\Leave\Providers;

use Illuminate\Support\ServiceProvider;

class LeaveServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'leave');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'leave');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerLeave();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerLeave()
    {
        $this->app->bind('leave', function($app) {
            return new Leave($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Leave', 'Lavalite\Leave\Facades\Leave');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('forms.leave', function ($app) {
            return $this->app->make('Forms\Leave\Leave');
        });

        // Bind Leave to repository
        $this->app->bind(
            'Forms\Leave\Interfaces\LeaveRepositoryInterface',
            \Forms\Leave\Repositories\Eloquent\LeaveRepository::class
        );        // Bind Category to repository
        $this->app->bind(
            'Forms\Leave\Interfaces\CategoryRepositoryInterface',
            \Forms\Leave\Repositories\Eloquent\CategoryRepository::class
        );

        $this->app->register(\Forms\Leave\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Forms\Leave\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and leave's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'forms.leave'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeLeave::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['forms.leave'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('forms/leave.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/leave')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/leave')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
